package meerkat;

import com.google.protobuf.*;
import meerkat.ProtobufsMessages.BasicMessage;
import meerkat.ProtobufsMessages.BulletinBoardMessage;
import meerkat.ProtobufsMessages.Tag;
import meerkat.bulletinboard.BulletinBoardClient;
import meerkat.comm.CommunicationException;
import meerkat.crypto.DigitalSignature;
import meerkat.protobuf.BulletinBoardAPI;

import java.io.IOException;
import java.io.OutputStream;
import java.util.List;
import java.util.Map;

/**
 *  Created by Vladimir Eliezer Tokarev on 1/8/2016.
 *  Gives the ability to manage voters information
 *  This object is synchronous (which meaning its method blocking)
 */
public class SimpleRegistry {

    protected DigitalSignature signatory;

    protected BulletinBoardClient communicator;

    /**
     * @param signatory implements the DigitalSignature interface
     * @param communicator implements the BulletinBoardClient interface
     */
    public SimpleRegistry(DigitalSignature signatory, BulletinBoardClient communicator){
        this.signatory = signatory;
        this.communicator = communicator;
    }

    /**
     * Adds new voter to the bulletin-board
     * @param voterID
     * @param personalData for example residence location
     * @throws throws CommunicationException
     * @return void
     */
    public void AddVoter(String voterID, String personalData) throws CommunicationException {
        Tag.Builder idTag = Tag.newBuilder();
        idTag.setContent(RegistryTagTypes.ID_TAG + " " + voterID);

        Tag.Builder voterEntryTag = Tag.newBuilder();
        voterEntryTag.setContent(RegistryTagTypes.VOTER_ENTRY_TAG);

        BasicMessage.Builder basicMessage = BasicMessage.newBuilder();
        basicMessage.setData(ByteString.copyFrom(personalData.getBytes()));
        basicMessage.addTag(idTag);
        basicMessage.addTag(voterEntryTag);

        communicator.postMessage(CreateBulletinBoardMessage(basicMessage), null);
    }

    /**
     * Adding given voter to given group
     * @param voterID
     * @param groupID
     * @throws CommunicationException
     * @return true if the adding action succeeded else return false
     */
    public void AddToGroup(String voterID, String groupID) throws CommunicationException {
        Tag.Builder idTag = Tag.newBuilder();
        idTag.setContent(RegistryTagTypes.ID_TAG + " " + voterID);

        Tag.Builder groupIDTag = Tag.newBuilder();
        groupIDTag.setContent(RegistryTagTypes.GROUP_ID_TAG + " " + groupID);

        Tag.Builder actionTag = Tag.newBuilder();
        actionTag.setContent(RegistryTagTypes.GROUP_ACTION_TAG + " " + RegistryTagTypes.ADD_TO_GROUP_TAG);

        BasicMessage.Builder basicMessage = BasicMessage.newBuilder();
        basicMessage.addTag(idTag);
        basicMessage.addTag(groupIDTag);
        basicMessage.addTag(actionTag);

        communicator.postMessage(CreateBulletinBoardMessage(basicMessage), null);
    }

    /**
     * Removes given voter from given group
     * @param voterID
     * @param groupID
     * @return  true if the removing action succeeded else return false
     * @throws CommunicationException
     */
    public void RemoveFromGroup(String voterID, String groupID) throws CommunicationException {
        Tag.Builder idTag = Tag.newBuilder();
        idTag.setContent(RegistryTagTypes.ID_TAG + " " + voterID);

        Tag.Builder groupIDTag = Tag.newBuilder();
        groupIDTag.setContent(RegistryTagTypes.GROUP_ID_TAG + " " + groupID);

        Tag.Builder actionTag = Tag.newBuilder();
        actionTag.setContent(RegistryTagTypes.GROUP_ACTION_TAG + " " + RegistryTagTypes.REMOVE_FROM_GROUP_TAG);

        BasicMessage.Builder basicMessage = BasicMessage.newBuilder();
        basicMessage.addTag(idTag);
        basicMessage.addTag(groupIDTag);
        basicMessage.addTag(actionTag);

        communicator.postMessage(CreateBulletinBoardMessage(basicMessage), null);
    }

    /**
     * Sets that the voter have voted
     * @param id
     * @return true if the set voted succeded else false
     * @throws CommunicationException
     */
    public void SetVoted(String id) throws CommunicationException {
        Tag.Builder idTag = Tag.newBuilder();
        idTag.setContent(RegistryTagTypes.ID_TAG + " " + id);

        Tag.Builder voteAction = Tag.newBuilder();
        voteAction.setContent(RegistryTagTypes.VOTE_ACTION_TAG);

        BasicMessage.Builder basicMessage = BasicMessage.newBuilder();
        basicMessage.addTag(idTag);
        basicMessage.addTag(voteAction);

        communicator.postMessage(CreateBulletinBoardMessage(basicMessage), null);
    }

    /**
     * Creates bulletin board message from basic message (with the signatures parts)
     * @param basicMessage  BasicMessage.Builder
     * @return BulletinBoardAPI.BulletinBoardMessage
     */
    private BulletinBoardAPI.BulletinBoardMessage CreateBulletinBoardMessage(BasicMessage.Builder  basicMessage) {
        BulletinBoardMessage.Builder bulletinBoardmessage = BulletinBoardMessage.newBuilder();
        Message encodableMessage = new Message() {
            @Override
            public Parser<? extends Message> getParserForType() {
                return null;
            }

            @Override
            public Builder newBuilderForType() {
                return null;
            }

            @Override
            public Builder toBuilder() {
                return null;
            }

            @Override
            public void writeTo(CodedOutputStream output) throws IOException {

            }

            @Override
            public int getSerializedSize() {
                return 0;
            }

            @Override
            public ByteString toByteString() {
                return null;
            }

            @Override
            public byte[] toByteArray() {
                return new byte[0];
            }

            @Override
            public void writeTo(OutputStream output) throws IOException {

            }

            @Override
            public void writeDelimitedTo(OutputStream output) throws IOException {

            }

            @Override
            public Message getDefaultInstanceForType() {
                return null;
            }

            @Override
            public List<String> findInitializationErrors() {
                return null;
            }

            @Override
            public String getInitializationErrorString() {
                return null;
            }

            @Override
            public Descriptors.Descriptor getDescriptorForType() {
                return null;
            }

            @Override
            public Map<Descriptors.FieldDescriptor, Object> getAllFields() {
                return null;
            }

            @Override
            public boolean hasOneof(Descriptors.OneofDescriptor oneof) {
                return false;
            }

            @Override
            public Descriptors.FieldDescriptor getOneofFieldDescriptor(Descriptors.OneofDescriptor oneof) {
                return null;
            }

            @Override
            public boolean hasField(Descriptors.FieldDescriptor field) {
                return false;
            }

            @Override
            public Object getField(Descriptors.FieldDescriptor field) {
                return null;
            }

            @Override
            public int getRepeatedFieldCount(Descriptors.FieldDescriptor field) {
                return 0;
            }

            @Override
            public Object getRepeatedField(Descriptors.FieldDescriptor field, int index) {
                return null;
            }

            @Override
            public UnknownFieldSet getUnknownFields() {
                return null;
            }

            @Override
            public boolean isInitialized() {
                return false;
            }
        }

        /**
         *  encode the basic  message
         *  create bulletin board message
         *  put the basic message in bulletin board message
         *  put the encoded messages signature into the bulletin board message
         *  return the bulletin board message
         */
    }

    /**
     * Requests all the groups that the given id voter is in
     * @param ID
     * @return list of groups ids (or names), if the method fails its empty
     * @throws CommunicationException
     */
    public List<String> GetGroups(String ID)  throws CommunicationException {
        /**
         *  Retrieve all List<BulletinBoardMessage> that contains this id
         *  creates new list of strings with the names of the group
         *  Returns the created list
         */
    }

    /**
     * Retreives list of strings that represents voter
     * @param id
     * @return list of strings (empty list if the lookup failed)
     * @throws CommunicationException
     */
    public List<String> GetPersonIDDetails(String id) throws CommunicationException {
        /**
         *      Retreive all List<BulletinBoardMessage> that contains this id
         *      search for message with  the wanted personal data
         *      Create list with personal data
         *      return the list
         */
    }
}
